using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Ink;
using System.IO;
using System.Xml;

namespace My_Ink_Project_C_Sharp
{
    public partial class MyInkProject : Form
    {
       
        InkOverlay myDrawingInk ;      //'object to hold drawing ink
        InkCollector myWritingInk ;    //    'object to hold writing ink
        Recognizers myRecognizers ;  // 'collection of different available recognizers

        public MyInkProject()
        {
            InitializeComponent();
        }

        private void MyInkProject_Load(object sender, EventArgs e)
        {
            
            myDrawingInk = new InkOverlay(picDrawingArea.Handle);
            myDrawingInk.Enabled = true;
            myDrawingInk.DefaultDrawingAttributes.Width = 85; //' thin ink

            myWritingInk = new InkCollector(picWritingArea.Handle);
            myWritingInk.Enabled = true;

            myRecognizers = new Recognizers();
         }

        private void btnRecog_Click(object sender, EventArgs e)
        {
            if (myRecognizers.Count == 0)
                MessageBox.Show("There are no handwriting recognizers installed.  You need to have at least one in order to run this sample.");
            else
            {
                lblDisplayText.Text = lblDisplayText.Text + " " + myWritingInk.Ink.Strokes.ToString();
                myWritingInk.Ink.DeleteStrokes(myWritingInk.Ink.Strokes);
                picWritingArea.Refresh();
             }
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            myDrawingInk.EditingMode = InkOverlayEditingMode.Ink;
        }

        private void btnErase_Click(object sender, EventArgs e)
        {
            myDrawingInk.EditingMode = InkOverlayEditingMode.Delete;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            myWritingInk.Ink.DeleteStrokes(myWritingInk.Ink.Strokes);
            lblDisplayText.Text = "";
            picWritingArea.Refresh();
        }

        private void btnSelect_Click(object sender, EventArgs e)
        {
            myDrawingInk.EditingMode = InkOverlayEditingMode.Select;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = "Ink Serialized Format files (*.isf)|*.isf";
            
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                FileStream myStream = new FileStream(saveFileDialog1.FileName, FileMode.Create, FileAccess.ReadWrite);
                XmlTextWriter xmlWriter = new XmlTextWriter(myStream, Encoding.UTF8);
                xmlWriter.WriteStartDocument();

                //    Write the beginning of the "data" element. This is 
                //    the opening tag to our data 
                xmlWriter.WriteStartElement("SerialisationOfInk");

                //  Get the base64 encoded ISF
                UTF8Encoding utf8 = new UTF8Encoding();  //converts the ink to a writable form
                Byte[] base64ISFBytes;
                String base64ISFString; 

                base64ISFBytes = myDrawingInk.Ink.Save(PersistenceFormat.Base64InkSerializedFormat);
                //   Ink.Save returns a null terminated byte array. The encoding of the null
                //   character generates a control sequence when it is UTF8 encoded. This
                //   sequence is invalid in XML. Therefore, it is necessary to remove the 
                //   null character before UTF8 encoding the array.
                //   The following loop finds the index of the first non-null byte in the byte 
                //   array returned by the Ink.Save method.
                int count;
                for( count = base64ISFBytes.Length - 1; count <= 0; count=-1){
                    if( base64ISFBytes[count] != 0)
                        break;
                }
                //    Convert the index into count by incrementing it.
                count += 1;
                //   Convert it to a String
                base64ISFString = utf8.GetString(base64ISFBytes, 0, count);

                // Write the ISF containing node to the XML
                xmlWriter.WriteElementString("DrawingInk", base64ISFString);
                xmlWriter.WriteEndElement();

                //  End the document
                xmlWriter.WriteEndDocument();

                xmlWriter.Close();
                myStream.Close();
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "Ink Serialized Format files (*.isf)|*.isf";
            if (openFileDialog1.ShowDialog() == DialogResult.OK){
                FileStream myStream = new FileStream(openFileDialog1.FileName, FileMode.Open, FileAccess.Read);
                UTF8Encoding utf8 = new UTF8Encoding();
                XmlDocument xmldoc = new XmlDocument();
                XmlNodeList xmlNodes;

                Ink inkLoaded = new Ink();
                xmldoc.Load(myStream);

                //Get the data in the ink node
                xmlNodes = xmldoc.GetElementsByTagName("DrawingInk");
                //load the ink into a new ink object
                //once an ink object has been "dirtied" it can never load ink again
                if (xmlNodes.Count != 0)
                    inkLoaded.Load(utf8.GetBytes(xmlNodes[0].InnerXml));
               
                //temporarily disable the ink collector and swap ink objects
                myDrawingInk.Enabled = false;
                myDrawingInk.Ink = inkLoaded;

                //Repaint the inkable region
                picDrawingArea.Refresh();
                myDrawingInk.Enabled = true;
                myStream.Close();
            }
        }

        private void btnBlack_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Color = Color.Black;
            
        }

        private void btnThin_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Width = 85;
        }

        private void btnMedium_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Width = 175;
        }

        private void btnThick_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Width = 300;
        }

        private void btnGreen_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Color = Color.Green;
        }

        private void btnYellow_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Color = Color.Yellow;
        }

        private void btnBlue_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Color = Color.Blue;
        }

        private void btnRed_Click(object sender, EventArgs e)
        {
            myDrawingInk.DefaultDrawingAttributes.Color = Color.Red;
        }
      
    }
}